@@ -34,14 +34,16 @@
  */
 
 /* Aliasing for compiler optimizations */
-#ifndef restrict
-	#ifdef __restrict
+#ifdef __cplusplus
+	#if defined(__GNUG__) || defined(__GNUC__) || defined(_MSC_VER) || defined(__clang__) || defined(__INTEL_COMPILER)
 		#define restrict __restrict
-	#elif defined(__restrict__)
-		#define restrict __restrict__
 	#else
-		#define restrict
+		#define restrict 
 	#endif
+#elif defined(_MSC_VER)
+	#define restrict __restrict
+#elif !defined(__STDC_VERSION__) || (__STDC_VERSION__ < 199901L)
+	#define restrict 
 #endif
 /* Note: MSVC is a special boy which does not allow 'restrict' in C mode,
    so don't move this piece of code down with the others, otherwise the
